package NET.worlds.scape;

import java.util.Enumeration;
import java.util.Random;
import java.util.Vector;

public class RandomizeRoom extends Action implements NonPersister {
    private boolean useSeed = true;
    private int seed = 0;
    private float maxCoordChange = 0.0f;
    private float maxAngleChange = 0.0f;
    
    public Persister trigger(Event event, Persister persister) {
        Random rng;
        if(useSeed) {
            rng = new Random(seed);
        } else {
            rng = new Random();
        }
        Room room = getOwner().getRoom();
        Enumeration contents = room.getContents();
        while(contents.hasMoreElements()) {
            Transform element = (Transform)contents.nextElement();
            randomize(element, rng);
        }
        Enumeration env = room.getEnvironment().getContents();
        while(env.hasMoreElements()) {
            Transform element = (Transform)env.nextElement();
            randomize(element, rng);
        }
        return null;
    }
    
    private void randomize(Transform t, Random rng) {
        if(t instanceof Drone) return;
        float deltaX = 2 * maxCoordChange * rng.nextFloat() - maxCoordChange;
        float deltaY = 2 * maxCoordChange * rng.nextFloat() - maxCoordChange;
        float deltaZ = 2 * maxCoordChange * rng.nextFloat() - maxCoordChange;
        t.moveBy(deltaX, deltaY, deltaZ);
        float yaw = 2 * maxAngleChange * rng.nextFloat() - maxAngleChange;
        float pitch = 2 * maxAngleChange * rng.nextFloat() - maxAngleChange;
        float roll = 2 * maxAngleChange * rng.nextFloat() - maxAngleChange;
        t.yaw(yaw);
        t.pitch(pitch);
        t.roll(roll);
    }
    
    public Object properties(int index, int offset, int mode, Object value) throws NoSuchPropertyException {
        Object ret = null;
        switch (index - offset) {
            case 0:
                if(mode == 0) {
                    ret = BooleanPropertyEditor.make(new Property(this, index, "Use seed"), "Do not use seed", "Use seed");
                } else if(mode == 1) {
                    ret = new Boolean(this.useSeed);
                } else if(mode == 2) {
                    this.useSeed = ((Boolean)value).booleanValue();
                }
                break;
            case 1:
                if(mode == 0) {
                    ret = IntegerPropertyEditor.make(new Property(this, index, "Seed"));
                } else if(mode == 1) {
                    ret = new Integer(this.seed);
                } else if(mode == 2) {
                    this.seed = ((Integer)value).intValue();
                }
                break;
            case 2:
                if(mode == 0) {
                    ret = FloatPropertyEditor.make(new Property(this, index, "Maximum coordinate change"));
                } else if(mode == 1) {
                    ret = new Float(this.maxCoordChange);
                } else if(mode == 2) {
                    this.maxCoordChange = ((Float)value).floatValue();
                }
                break;
            case 3:
                if(mode == 0) {
                    ret = FloatPropertyEditor.make(new Property(this, index, "Maximum angle change per axis"));
                } else if(mode == 1) {
                    ret = new Float(this.maxAngleChange);
                } else if(mode == 2) {
                    this.maxAngleChange = ((Float)value).floatValue();
                }
                break;
            default:
                ret = super.properties(index, offset + 4, mode, value);
        }
        return ret;
    }
}